<?php

namespace Storage;

class Cookie {

    public array $options = [
        'expires'  => 0,
        'path'     => null,
        'domain'   => null,
        'secure'   => false,
        'httponly' => true,
        'samesite' => "Strict",
    ];

    protected ? string $secureHash;

    public function __construct(array $options, ? string $secureHash) {
        $this->options = $options;
        $this->secureHash = $secureHash;
    }

    /**
     * Sets a cookie with the given parameters.
     * @param   string   cookie name or array of config options
     * @param   string   cookie value
     * @param   integer  number of seconds before the cookie expires
     * @param   string   URL path to allow
     * @param   string   URL domain to allow
     * @param   boolean  HTTPS only
     * @param   boolean  HTTP only (requires PHP 5.2 or higher)
     * @return  boolean
     */
    public function set(
        ?string $name,
        ?string $value = null,
        ?int $expires = null,
        ?string $path = null,
        ?string $domain = null,
        ?bool $secure = null,
        ?bool $httponly = null,
        ?string $samesite = null,
        ?bool $raw = false,
        ?bool $skipHash = false
    ) {
        if ( headers_sent() ) {
            return false;
        }

        $expires = ! $expires ? 0 : time() + (int) $expires;

        $_COOKIE[$name] = $value;

        $options = [
            'expires'  => $this->options['expires']  ?? ( $expires ?: 0 ),
            'path'     => $this->options['path']     ?? ( $path ?: "" ),
            'domain'   => $this->options['domain']   ?? ( $domain ?: "" ),
            'secure'   => $this->options['secure']   ?? ( $secure ?: false ),
            'httponly' => $this->options['httponly'] ?? ( $httponly ?: false ),
            'samesite' => $this->options['samesite'] ?? ( $samesite ?: "" ),
        ];

        if ( $value && ! $skipHash ) {
            $value = sha1($this->secureHash . $value . $this->secureHash) . "|$value";
        }

        return $raw ? setrawcookie($name, $value ?: "", $options) : setcookie($name, $value ?: "", $options);
    }

    /**
     * Fetch a cookie value, using the Input library.
     * @param   string   cookie name
     * @param   mixed    default value
     * @return  string
     */
    public function get(string $key, $default = null, $skipHash = false)
    {
        $key = $this->normalizeKey($key);

        if ( ! $this->has($key) ) {
            return $default;
        }

        if ( $this->secureHash && ! $skipHash ) {
            if (strpos($_COOKIE[$key], '|') === false) {
                throw new Exception\CookieInvalidValueException("A cookie value was found which contains no secure hash while a key was provided.");
            }

            list($hash, $value) = explode('|', $_COOKIE[$key], 2);

            if (! $this->isSecure($hash, $value)) {
                $this->delete($key);

                throw new Exception\CookieInvalidSecureHashException("Invalid cookie hash value given.");
            }

            return $value;
        }

        return $_COOKIE[$key];
    }

    public function has($key) : bool
    {
        $key = $this->normalizeKey($key);

        return array_key_exists($key, $_COOKIE ?? []);
    }

    /**
     * Nullify and unset a cookie.
     * @param   string   cookie name
     * @param   string   URL path
     * @param   string   URL domain
     * @return  boolean
     */
    public function delete(string $key, ?string $path = null, ?string $domain = null) {
        $key = $this->normalizeKey($key);

        if ( ! $this->has($key) ) {
            return false;
        }

        unset( $_COOKIE[$key] );

        return $this->set($key, '', -86400, $path ?: ( $this->options['path'] ?? "" ), $domain ?: ( $this->options['domain'] ?? "" ), $this->options['secure'] ?? false, $this->options['httponly'] ?? false, null, false, true);
    }

    public function isSecure($hash, $value) : bool
    {
        return sha1($this->secureHash . $value . $this->secureHash) === $hash;
    }

    public function __invoke(...$arguments)
    {
        switch(count($arguments)) {
            case 1: return $this->get(...$arguments);
            case 2: return $this->set(...$arguments);
        }
    }

    public function __set($key, $value)
    {
        return static::set($key, $value);
    }

    public function __get($key)
    {
        return static::get($key);
    }

    public function __isset($key)
    {
        return array_key_exists($key, $_COOKIE);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }

    protected function normalizeKey(string $key) : string
    {
        return str_replace(['.', ' ',], '_', $key);
    }
}
