<?php

namespace Storage;

use session_name, session_id, session_start, session_destroy, session_save_path,
    session_regenerate_id, session_cache_limiter, session_get_cookie_params,
    session_set_cookie_params, session_status, session_write_close, time, array_key_exists;

class Session
{
    public static function get($key, $default = null)
    {
        return static::has($key) ? $_SESSION[$key] : $default;
    }

    public static function set($key, $value)
    {
        return $_SESSION[$key] = $value;
    }

    public static function delete($key)
    {
        unset($_SESSION[$key]);
    }

    public static function has($key) : bool
    {
        return array_key_exists($key, $_SESSION ?? []);
    }

    public static function clearAll() : void
    {
        $_SESSION = [];
    }
    
    public static function all() : array
    {
        return $_SESSION;
    }

    public static function start(array $options, Cookie $cookie, string $savePath) : void
    {
        session_save_path($savePath);

        if ( session_status() === PHP_SESSION_ACTIVE ) {
            return;
        }

        $params = session_get_cookie_params();

        $params = [
            'lifetime' => $options['lifetime'] ?? $params['lifetime'],
            'path'     => $options['path']     ?? $params['path'],
            'domain'   => $options['domain']   ?? $params['domain'],
            'secure'   => $options['secure']   ?? $params['secure'],
            'httponly' => $options['httponly'] ?? $params['httponly'],
            'samesite' => $options['samesite'] ?? $params['samesite'] ?? "",
        ];

        if ( version_compare(PHP_VERSION, '7.3.0') >= 0 ) {
            session_set_cookie_params($params);
        }
        else {
            session_set_cookie_params( ...array_values(array_slice($params, 0, 5)) );
        }
        
        session_name($options['name']);

        if ( isset($options['cache_limiter']) ) {
            session_cache_limiter($options['cache_limiter'] ?? 'nocache');
        }
        session_start();
    }
    
    public static function stop() {
        session_write_close();
    }

    public static function regenerate()
    {
        if ( session_status() === PHP_SESSION_ACTIVE ) {
            session_regenerate_id(true);
        }
    }

    public static function destroy()
    {
        static::clearAll();

        if ( ini_get("session.use_cookies") ) {
            ( new Cookie([], null) )->delete(session_name(), $params['path'] ?? "", $params['domain'] ?? "");
        }

        if ( session_status() === PHP_SESSION_ACTIVE ) {
            session_destroy();
        }
    }

    public function __invoke(...$arguments)
    {
        switch(count($arguments)) {
            case 1: return static::get(...$arguments);
            case 2: return static::set(...$arguments);
        }
    }

    public function __set($key, $value)
    {
        return static::set($key, $value);
    }

    public function __get($key)
    {
        return static::get($key);
    }

    public function __isset($key)
    {
        return array_key_exists($key, $_SESSION);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }
}
